
> module Communication where

> import Network
> import Robot
> import Numeric
> import IO
> import Map

> import Array
> import Paths

> arraymap f a = array (bounds a) [(i, f (a!i))| i<-range (bounds a)] 

> run (initializeRobot,doRobot,describeState) host port = withSocketsDo $ do
>     h<-connectTo host (PortNumber (fromIntegral port))
>     hSetBuffering h LineBuffering
>     hPutStr h "Player\n"
>     b <- readBoard h
>     putStr (showMap b)
>     (id, cap, funds) <- readStats h
>     putStr ((show (id,cap,funds))++"\n")
>     execute (initializeRobot (b::Map) ((id::Int),(cap::Int),(funds::Int))) h
>     return ()
>   where execute s h = do events <- readParseLine parseServerEvents h
>                          putStr ((show events)++"\n")
>                          packagesHere <- readParseLine parsePackageList h
>                          putStr ((show packagesHere)++"\n")
>                          (ns, c) <- return (doRobot s events packagesHere)
>                          putStr (formatCommand c)
>                          hPutStr h (formatCommand c)
>                          putStr (describeState ns)
>                          execute ns h



> readSize h = do
>     whline <- hGetLine h
>     return (case map read $ words whline of [w,h] -> (w::Int,h::Int))

> readBoard h = do
>     (width,height) <- readSize h
>     boardLines <- sequence [hGetLine h | i<-[1..height]]
>     return (mapFromLines boardLines)

> readStats h = do
>     line <- hGetLine h
>     return (case map read $ words line of [id, cap, funds] -> (id::Int, cap::Int, funds::Int))

> readParseLine f h = do l <- hGetLine h
>                        return (f l)


> parseServerEvents l = parseWords (words l) 0
>     where parseWords [] _ = []
>           parseWords (x:xs) _ | (head x)=='#' = (RobotAlive n):parseWords xs n
>                               where n = (read (tail x))
>           parseWords ("E":xs) n = (RobotMoved n East):(parseWords xs n)
>           parseWords ("W":xs) n = (RobotMoved n West):(parseWords xs n)
>           parseWords ("N":xs) n = (RobotMoved n North):(parseWords xs n)
>           parseWords ("S":xs) n = (RobotMoved n South):(parseWords xs n)
>           parseWords ("P":id:xs) n = (RobotPickedUp n (read id)):(parseWords xs n)
>           parseWords ("D":id:xs) n = (RobotDropped n (read id)):(parseWords xs n)
>           parseWords ("X":x:"Y":y:xs) n = (RobotSentTo n ((read x),(read y))):(parseWords xs n)
>           parseWords w _ = error $ "Don't understand "++(unwords w)

> parsePackageList l = parseWords (words l)
>     where parseWords [] = []
>           parseWords (id:dx:dy:wt:s) = (read id, (read dx, read dy), read wt):(parseWords s)
>           parseWords w = error $ "Don't understand "++(unwords w)

> pasteShows = unwords.(map show)

> formatCommand (Move bid _ North) = (show bid)++" Move N\n"
> formatCommand (Move bid _ South) = (show bid)++" Move S\n"
> formatCommand (Move bid _ East) = (show bid)++" Move E\n"
> formatCommand (Move bid _ West) = (show bid)++" Move W\n"
> formatCommand (Pick bid _ idl ) = (show bid)++" Pick "++(pasteShows idl)++"\n"
> formatCommand (Drop bid _ idl ) = (show bid)++" Drop "++(pasteShows idl)++"\n"




> extremelyStupidInit map (id,cap,funds) = id -- don't care about map
> extremelyStupidDo id events pkgs = (id, Move 50 id North)
> extremelyStupidRobot = (extremelyStupidInit, extremelyStupidDo)
